package l08_exceptions;

//	The 'finally' block always executes when the try block exits. This ensures
//	that the 'finally' block is executed even if an unexpected exception occurs.
//	But finally is useful for more than just exception handling — it allows the
//	programmer to avoid having cleanup code accidentally bypassed by a return,
//	continue, or break. Putting cleanup code in a 'finally' block is always a good
//	practice, even when no exceptions are anticipated.

public class FinallyCleanUp {

	private static class Foo {
		static void doSomething() {
			System.out.println("In doSomething()");
			throw new RuntimeException();
		}
		static void cleanup() {
			System.out.println("In cleanup()");
		}
	}

	private static void withoutFinally() {
		System.out.println("--- In withoutFinally()");
		try {
			Foo.doSomething();
			Foo.cleanup();
		} catch (RuntimeException e) {
			System.out.println("Exception caught");
		}
	}

	private static void withFinally() {
		System.out.println("--- In withFinally()");
		try {
			Foo.doSomething();
		} catch (RuntimeException e) {
			System.out.println("Exception caught");
		} finally {
			Foo.cleanup();
		}
	}

	public static void main(String ... args) {
		withoutFinally();
		System.out.println();
		withFinally();
	}
}

