package l08_exceptions;

//	public interface AutoCloseable

//	An object that may hold resources (such as file or socket handles) until it
//	is closed. The close() method of an AutoCloseable object is called
//	automatically when exiting a try-with-resources block for which the object
//	has been declared in the resource specification header. This construction
//	ensures prompt release, avoiding resource exhaustion exceptions and errors
//	that may otherwise occur.

//	public void close()

//	Closes this resource, relinquishing any underlying resources.

//	This is translated by the compiler to a sequence of nested 
//	try/catch/finally blocks. In each of these nested blocks
//	subsequent variable is managed. Variables are closed in the opposite 
//	order of their creation

public class MyContextManagerMain {

	private static class MyContextManager implements AutoCloseable {
		public MyContextManager() {
			System.out.println("Creating MyContextManager!");
		}

		public void doSomething() {
			System.out.println("Doing something... :)");
		}

		@Override
		public void close() {
			System.out.println("Somebody is closing me!");
		}
	}

	public static void main(String[] args) {

		try (MyContextManager manager = new MyContextManager()) {
			manager.doSomething();
		}
	}
}

